// file: status_disp.cpp
//
// Subscribe to the datastream status from the relayboard
//  and display on stdout.
//
// compile with:
//  $ gcc -o status_disp status_disp.cpp -llcm
//
// On a system with pkg-config, you can also use:
//  $ gcc -o status_disp status_disp.cpp `pkg-config --cflags --libs lcm`

#include <stdio.h>

#include <lcm/lcm-cpp.hpp>
#include "dataspeed_pds_lcm/status_t.hpp"

enum {
    RELAY_STATUS_OFF = 0,
    RELAY_STATUS_ON,
    RELAY_STATUS_NO_LOAD,
    RELAY_STATUS_BAD_RELAY,
    RELAY_STATUS_BAD_FUSE,
    RELAY_STATUS_OVER_CURRENT,
    RELAY_STATUS_BLINK,
};

enum {
    MODE_AUTO = 0,
    MODE_MANUAL,
    MODE_VALET,
};

enum {
    ID_MASTER = 0,
    ID_SLAVE1,
    ID_SLAVE2,
    ID_SLAVE3,
};

enum {
    INV_STATUS_OFF = 0,
    INV_STATUS_ON,
};

class Handler 
{
    public:
        ~Handler() {}

        std::string getStatusText(int status)
        {
            switch (status)
            {
                case RELAY_STATUS_OFF: return "off";
                case RELAY_STATUS_ON: return "on";
                case RELAY_STATUS_NO_LOAD: return "no load";
                case RELAY_STATUS_BAD_RELAY: return "bad relay";
                case RELAY_STATUS_BAD_FUSE: return "bad fuse";
                case RELAY_STATUS_OVER_CURRENT: return "over current";
                case RELAY_STATUS_BLINK: return "blink";
            }
            return "";
        }

        std::string getModeText(int mode)
        {
            switch (mode)
            {
                case MODE_AUTO: return "auto";
                case MODE_MANUAL: return "manual";
                case MODE_VALET: return "valet";
            }
            return "";
        }

        std::string getIdText(int id)
        {
            switch (id)
            {
                case ID_MASTER: return "Master";
                case ID_SLAVE1: return "Slave 1";
                case ID_SLAVE2: return "Slave 2";
                case ID_SLAVE3: return "Slave 3";
            }
            return "";
        }

        std::string getInverterText(int status)
        {         
            switch (status)
            {
                case INV_STATUS_OFF: return "off";
                case INV_STATUS_ON: return "on";
            }
            return "";
        }

        void onStatusMessage(const lcm::ReceiveBuffer* rbuf,
                const std::string& chan, 
                const dataspeed_pds_lcm::status_t* msg)
        {
            printf("Status of %s:\n", getIdText(msg->unit_id).c_str());
            printf("  Voltage       = %.2f V\n", msg->voltage);
            printf("  Ignition1     = %.2f V\n", msg->ignition[0]);
            printf("  Ignition2     = %.2f V\n", msg->ignition[1]);
            printf("  Ignition3     = %.2f V\n", msg->ignition[2]);
            printf("  Board temp    = %.2f °C\n", msg->board_temp);
            printf("  Thermocouple  = %.2f °C\n", msg->thermocouple);
            printf("  Total current = %.2f A\n", msg->total_current);
            printf("  Mode          = %s\n", getModeText(msg->mode).c_str());
            printf("  Inverter      = %s\n", getInverterText(msg->inverter_status).c_str());
            int i;
            for(i = 0; i < 12; i++)
            {
                printf("  Current%d = %.3f A\n", i+1, msg->current[i]);
                printf("   Status%d = %s\n",   i+1, getStatusText(msg->status[i]).c_str());
            }
        }
};

int main(int argc, char** argv)
{
    lcm::LCM lcm("udpm://225.0.0.0:7667?ttl=1");

    if(!lcm.good())
        return 1;

    Handler handlerObject;
    lcm.subscribe("STATUS", &Handler::onStatusMessage, &handlerObject);

    while(0 == lcm.handle());

    return 0;
}

